import React, { useCallback, useContext, useMemo, useState } from 'react'

import Modal from '../components/Modal'
import { generateServerUrl, isAccessTokenValid, nop } from '../utils'
import { AuthConfig, IAuthContext, ServerEndpointType, ERRORS } from './models'
import { getRefreshToken } from '../actions'
import useRefreshToken from '../hooks/useRefreshToken'
import useUser from '../hooks/useUser'
import Authenticate from '../components/Authenticate'

const AuthContext = React.createContext<IAuthContext>({
  login: nop,
  logout: nop,
  register: nop,
  user: {},
  isUserLoading: false,
  isAuthenticated: false,
  refreshToken: nop,
})

export const AuthProvider: React.FC<AuthConfig> = (props) => {
  const {
    children,
    clientID,
    serverUrl,
    postRegisterRedirectUrl = null,
    postLogoutRedirectUrl = null,
    postLoginRedirectUrl = null,
    onError = nop,
  } = props

  const [isAuthenticated, setIsAuthenticated] = useState(() => isAccessTokenValid())
  const [loginModal, setLoginModal] = useState(false)

  const { isUserLoading, user } = useUser({ serverUrl, clientID, onError })

  const refreshToken = useCallback(() => {
    return getRefreshToken({ serverUrl, clientID })
      .then((data) => {
        if (data) {
          setIsAuthenticated(true)
        }
        return 'success'
      })
      .catch((error) => {
        setIsAuthenticated(false)
        onError(ERRORS.REFRESH_TOKEN, error)
        return error
      })
  }, [serverUrl, clientID, setIsAuthenticated, onError])

  useRefreshToken({ setIsAuthenticated, refreshToken, serverUrl, clientID })

  const login = useCallback(() => {
    const url = generateServerUrl({
      serverEndpoint: ServerEndpointType.login,
      serverUrl,
      queryParams: {
        client_id: clientID,
        redirect_uri: postLoginRedirectUrl ?? window.location.href,
      },
    })

    window.location.assign(url)
  }, [clientID, serverUrl, postLoginRedirectUrl])

  const logout = useCallback(() => {
    const url = generateServerUrl({
      serverEndpoint: ServerEndpointType.logout,
      serverUrl,
      queryParams: {
        client_id: clientID,
        redirect_uri: postLogoutRedirectUrl ?? window.location.href,
      },
    })

    window.location.assign(url)
  }, [clientID, serverUrl, postLogoutRedirectUrl])

  const register = useCallback(() => {
    const url = generateServerUrl({
      serverEndpoint: ServerEndpointType.register,
      serverUrl,
      queryParams: {
        client_id: clientID,
        redirect_uri: postRegisterRedirectUrl ?? window.location.href,
      },
    })

    window.location.assign(url)
  }, [clientID, postRegisterRedirectUrl, serverUrl])

  const providerValue = useMemo(
    () => ({
      login,
      logout,
      register,
      isAuthenticated,
      isUserLoading,
      user,
      refreshToken,
      Authenticate,
    }),
    [login, logout, register, isAuthenticated, isUserLoading, refreshToken, user],
  )

  return (
    <AuthContext.Provider value={providerValue}>
      {loginModal && <Modal setIsOpen={setLoginModal} login={login} logout={logout} />}
      {children}
    </AuthContext.Provider>
  )
}

export const useAuth = () => useContext(AuthContext)
